
local nxml = dofile("mods/ArenaForMonster/nxml.lua")
dofile("mods/ArenaForMonster/scripts/list/biome_list.lua")
local biomes_list = get_biome_list()
local biome_abstract_path = "mods/ArenaForMonster/abstract_file/"
local ban_list = {
    "$animal_thundermage_big",--will crash the game at death
}
local function is_banned(file)
    for _,ban_file in pairs(ban_list) do
        if ban_file == file then
            return true
        end
    end
    return false
end
local function get_first_table_pos(lua_text)
    local table_start = string.find(lua_text,"{")

    if table_start ~= nil then
        local brackets_counter = 1
        local table_end
        for i=table_start+1,string.len(lua_text) do
            if string.sub(lua_text,i,i) == "{" then
                brackets_counter = brackets_counter+1
                --print("brackets_counter + 1,current counter:"..tostring(brackets_counter))
            elseif string.sub(lua_text,i,i) == "}" then
                brackets_counter = brackets_counter-1
                --print("brackets_counter - 1,current counter:"..tostring(brackets_counter))
            end
            if brackets_counter == 0 then
                table_end = i
                break
            end
        end
        if table_end ~= nil then
            return table_start,table_end
        else
            print("lua text no finish")
            --print(string.sub(lua_text,table_start,string.len(lua_text)-table_start))
            return nil
        end
    else
        print("table don't have content")
        return nil
    end
end

local function get_table_by_pattern(lua_text,pattern)
    local pattern_start  = string.find(lua_text,pattern)
    if pattern_start == nil then
        print("No such table:"..pattern)
        return nil
    end
    local new_text = string.sub(lua_text,pattern_start,string.len(lua_text)-pattern_start)
    local table_start,table_end =  get_first_table_pos(new_text)

    if table_start == nil then
        print("failed when getting "..pattern.." table")
        return nil
    end

    return string.sub(new_text,1,table_end)
end

local function filepath2name(filepath)
    local name = ""
    for i=string.len(filepath),1,-1 do
        if string.sub(filepath,i,i)=="/" then
            name = string.sub(filepath,i,string.len(filepath)-4)
        end
    end
    return name
end
local function get_monster_data(filepath)

    local monster_xml_text = ModTextFileGetContent(filepath)
    if monster_xml_text == nil then
        print("data of "..filepath.." is nil!")
        return
    end
    local monster_data = {
        is_creature = false,
        name = "???",
        sprite = {
            filepath = nil,
            offset_x = nil,
            offset_y = nil,
        },
        value = nil,
        filepath = filepath,
    }

    local xml = nxml.parse(monster_xml_text)
    local sprite_loaded = false
    for element in xml:each_child() do
        if element.name == "Base" then
            if element.attr.file ~= nil then
                --print(filepath.." has a base file:"..element.attr.file)
                local data = get_monster_data(element.attr.file)
                if data ~= nil then
                    monster_data = data
                    monster_data.filepath = filepath
                end
            end
            for component in element:each_child() do
                if component.name == "SpriteComponent" or component.name == "PhysicsImageShapeComponent" then
                    local is_main_sprite = true
                    if component.attr._tags ~= nil then
                        if component.attr._tags == "character" then
                            --special check for acid flower in rain forest
                            is_main_sprite = false
                        end
                    end
                    if sprite_loaded == true then
                        is_main_sprite = false
                    end

                    if string.sub( component.attr.image_file,1,string.len("data/enemies_gfx")) ~= "data/enemies_gfx" then
                        is_main_sprite = false
                    end

                    if is_main_sprite or monster_data.sprite.filepath == nil then
                        local sprite = {
                            filepath = component.attr.image_file,
                            offset_x = tonumber(component.attr.offset_x) ,
                            offset_y = tonumber(component.attr.offset_y) ,
                            --rect_animation = component.attr.rect_animation
                        }
                        if is_main_sprite then
                            sprite_loaded = true
                        end
                        monster_data.sprite = sprite
                    end
                    
                elseif component.name == "DamageModelComponent" then
                    monster_data.value = 10 * component.attr.hp
                elseif component.name == "AnimalAIComponent" then
                    monster_data.is_creature = true
                end
            end
        end
    end

    local xml = nxml.parse(monster_xml_text)

    if xml.attr.name ~= nil then
        monster_data.name = xml.attr.name
    end

    local sprite_loaded = false
    for component in xml:each_child() do
        if component.name == "SpriteComponent" or component.name == "PhysicsImageShapeComponent" then
            local is_main_sprite = true
            if component.attr._tags ~= nil then
                if component.attr._tags == "character" then
                    is_main_sprite = false
                end
            end
            if sprite_loaded == true then
                is_main_sprite = false
            end

            if string.sub( component.attr.image_file,1,string.len("data/enemies_gfx")) ~= "data/enemies_gfx" then
                is_main_sprite = false
            end

            if is_main_sprite or monster_data.sprite.filepath == nil then
                local sprite = {
                    filepath = component.attr.image_file,
                    offset_x = tonumber(component.attr.offset_x) ,
                    offset_y = tonumber(component.attr.offset_y) ,
                    --rect_animation = component.attr.rect_animation
                }
                if is_main_sprite then
                    sprite_loaded = true
                end
                monster_data.sprite = sprite
            end
        elseif component.name == "DamageModelComponent" then
            monster_data.value = 10 * component.attr.hp
        elseif component.name == "AnimalAIComponent" then
            monster_data.is_creature = true
        end
    end

    if is_banned(monster_data.name) then
        print(monster_data.name.."is banned. file path:"..filepath)
        return nil 
    end

    if monster_data.value == nil then
        print(filepath.." don't have damage model component!")
        return nil 
    elseif monster_data.is_creature == false then
        print(filepath.." is not a creature!")
        return nil 
    else
        return monster_data;
    end

end

local function generate_filepath_list(lua_filepath)
    g_small_enemies = nil
    g_big_enemies = nil
    g_unique_enemy = nil 
    dofile(lua_filepath)
    local monster_list = {}

    local function insert_monster_list(data)
        if type(data) ~= "table" then return end
        local filepath
        if data.entity ~= nil then
            filepath = data.entity
        elseif data.entities ~= nil then
            filepath = data.entities[1]
        end

        if type(filepath) == "string" then
            if filepath == "" then
                return
            end
            for _,file in pairs(monster_list) do
                if file == filepath then
                    return
                end
            end
            table.insert(monster_list,filepath)
            return
        end
    end

    if g_small_enemies~=nil then
        for _,data in pairs(g_small_enemies) do
            --print(tostring(#g_small_enemies).." enemies in g_small_enemie")
            insert_monster_list(data)
        end
    end
    if g_big_enemies~= nil then
        for _,data in pairs(g_big_enemies) do
            insert_monster_list(data)
        end
    end
    if g_unique_enemy ~= nil then
        for _,data in pairs(g_unique_enemy) do
            insert_monster_list(data)
        end
    end

    return monster_list
end

local table_name_list = {
    "g_small_enemies",
    "g_big_enemies",
    "g_unique_enemy",
}

local overall_biome_data ={}
--get monster in each biome
for biome_name, biome_filepath in pairs(biomes_list) do
    --print("loading "..biome_name.." from ".. biome_filepath)
    local lua_text = ModTextFileGetContent(biome_filepath)
    local final_table_text = ""
    for _,table_name in pairs(table_name_list) do
        local table_text = get_table_by_pattern(lua_text,table_name)
        if table_text == nil then
            print("Didn't read "..table_name.." from "..biome_name)
        else
            --print(table_text)
            final_table_text = final_table_text..table_text.. [[
            ]]
        end
    end
    --print(small_enemies_text)
    ModTextFileSetContent(biome_abstract_path..biome_name..".lua",final_table_text)
    local biome_monster_filepath_list = generate_filepath_list(biome_abstract_path..biome_name..".lua")
    local biome_monster_data = {
        name = biome_name,
        --on = false,
        --type = "monster_list",
        --selected_index = 0,
        content = {}
    }
    for _,filepath in pairs(biome_monster_filepath_list) do
        local monster_data = get_monster_data(filepath)
        if monster_data ~= nil then
            table.insert(biome_monster_data.content, monster_data)
        else
            print("Didnt add "..filepath.." of "..biome_name)
        end
    end
    print(biome_name.." load completed, "..tostring(#biome_monster_data.content).." monster loaded")
    table.insert( overall_biome_data,biome_monster_data)
    --overall_biome_data[biome_name] = biome_monster_data
end

return overall_biome_data;